//
// Minimal NDS project.
//
//! \file main.cpp
//! \author J Vijn
//! \date 20091119 - 20091119
//
/* === NOTES ===
	Base:				 53.3k
	Malloc:				 56.5k
	Std new:			117k
	Own new:			 56.5k
	vector:				117k
	printf:				 82.0k	
	std new+printf		124k
	own new + vector	 57.0k
*/

#include <vector>
#include <stdio.h>

#include <nds.h>

using namespace std;

// --------------------------------------------------------------------
// Defines for conditional compilation
// --------------------------------------------------------------------

#define USE_MALLOC	0		//!< Compile with malloc.
#define USE_STD_NEW	1		//!< Compile with standard new.
#define USE_OWN_NEW	0		//!< Compile with own new.
#define USE_VECTOR	0		//!< Use vector.push_back.
#define USE_STRING	0		//!< Use string (defines 1 string).
#define USE_PRINTF	0		//!< Use single printf statement.

// --------------------------------------------------------------------
// Tests
// --------------------------------------------------------------------

// Using nothing:

#if USE_MALLOC
// Cost of malloc/free : 
void test_malloc()
{
	u8 *ptr= (u8*)malloc(8);
	free(ptr);
}
#endif

#if USE_STD_NEW
// Cost of standard new/delete : 
void test_std_new()
{
	u8 *ptr= new u8[8];
	delete[] ptr;
}
#endif

#if USE_OWN_NEW
// Cost of user defined new/delete : 

void* operator new(size_t size)		{	return malloc(size);	}

void operator delete(void *p)		{	free(p);				}

void* operator new[](size_t size)	{	return malloc(size);	}

void operator delete[](void *p)		{	free(p);				}

//extern "C" void __assert(const char * , const char * , int ) {	}

void test_own_new()
{
	u8 *ptr= new u8[8];
	delete[] ptr;
}
#endif

#if USE_VECTOR
// Cost of vector.push_back : 
void test_vector()
{
	vector<int> v;
	v.push_back(42);
}
#endif

#if USE_STRING
// Cost of string : 
void test_string()
{
	string s;
	s= "oh hai!";
}
#endif


#if USE_PRINTF
// Cost of printf : 
void test_printf()
{
	printf("oh hai!");
}
#endif

int main(void)
{	
	while(1) ;

	return 0;
}
